/*
 * Copyright 2006 Takahiro Nakamura.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package woolpack.validator;


/**
 * キー、インデックス、メッセージ、値を保持するオブジェクトです。
 * 
 * @author nakamura
 * 
 */
public class AddressedMessage {
	private String key;
	private int index;
	private String message;
	private Object value;

	public AddressedMessage() {
	}

	public AddressedMessage(
			final String key,
			final int index,
			final String message,
			final Object value) {
		this.key = key;
		this.index = index;
		this.message = message;
		this.value = value;
	}

	public String getKey() {
		return key;
	}
	public void setKey(final String key) {
		this.key = key;
	}
	public int getIndex() {
		return index;
	}
	public void setIndex(final int index) {
		this.index = index;
	}
	public String getMessage() {
		return message;
	}
	public void setMessage(final String message) {
		this.message = message;
	}
	public Object getValue() {
		return value;
	}
	public void setValue(final Object value) {
		this.value = value;
	}

	@Override
	public String toString() {
		return new StringBuilder()
		.append("AddressedMessage:key=")
		.append(key)
		.append(",index=")
		.append(index)
		.append(",message=")
		.append(message)
		.append(",value=")
		.append(value)
		.toString();
	}

	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + index;
		result = PRIME * result + ((key == null) ? 0 : key.hashCode());
		result = PRIME * result + ((message == null) ? 0 : message.hashCode());
		result = PRIME * result + ((value == null) ? 0 : value.hashCode());
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final AddressedMessage other = (AddressedMessage) obj;
		if (index != other.index) {
			return false;
		}
		if (key == null) {
			if (other.key != null) {
				return false;
			}
		} else if (!key.equals(other.key)) {
			return false;
		}
		if (message == null) {
			if (other.message != null) {
				return false;
			}
		} else if (!message.equals(other.message)) {
			return false;
		}
		if (value == null) {
			if (other.value != null) {
				return false;
			}
		} else if (!value.equals(other.value)) {
			return false;
		}
		return true;
	}
}
